using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Text;
using System.Windows.Forms;
using ManagedSENS;

namespace pcNanny
{
	public partial class Form1 : Form
	{
		private Timer timer1;
		private UserList users = null;
		private GlobalKeyboardHook gkh = null;

		public Form1()
		{
			InitializeComponent();

			// Define the border style of the form to a dialog box.
			this.FormBorderStyle = FormBorderStyle.FixedDialog;
			this.StartPosition = FormStartPosition.CenterScreen;
			this.Text = "pcNanny";

			// Get rid of control boxes
			this.ControlBox = false;
			this.MaximizeBox = false;
			this.MinimizeBox = false;

			// Disable close window unless ending session
			this.Closing += new System.ComponentModel.CancelEventHandler(this.OnClosingEvent);

			// setup for double buffering
			this.SetStyle(ControlStyles.AllPaintingInWmPaint | ControlStyles.Opaque, true); 
			this.Size = new Size(300, 100);

			// setup a one second timer tick for update
			this.timer1 = new System.Windows.Forms.Timer();
			this.timer1.Enabled = true;
			this.timer1.Interval = 1000;
			this.timer1.Tick += new System.EventHandler(this.OnTimerTick);

			// load the users
            users = new UserList();

            UserInfo me = users.FindByName(Environment.UserName);
            Assert.Test(me != null);

            // install global keyboard hook
            gkh = new GlobalKeyboardHook();
            gkh.KeyPress += new KeyPressEventHandler(me.KeyPress);

            // dd/mm/yyyy hh:mm:ss AM/PM ==>
			me.Log("\nLogin on " + DateTime.Now.ToShortDateString().ToString() + " " +
				DateTime.Now.ToLongTimeString().ToString() + "\n");

            //
			// Subscribe to the SensLogon events
			//
			SensLogon.DisplayLock += new SensLogonEventHandler(SensLogon_DisplayLock);
            SensLogon.DisplayUnlock += new SensLogonEventHandler(SensLogon_DisplayUnlock);
            SensLogon.StartScreenSaver += new SensLogonEventHandler(SensLogon_StartScreenSaver);
            SensLogon.StopScreenSaver += new SensLogonEventHandler(SensLogon_StopScreenSaver);
            //SensLogon.Logon += new SensLogonEventHandler(SensLogon_Logon);
            //SensLogon.Logoff += new SensLogonEventHandler(SensLogon_Logoff);

			Console.WriteLine("Registered For SENS Events");

            // adjust form height
            int Height = 60 + users.Count * 60;
			if (UserInfo.IsAdmin())
            {
                button1.Location = new Point(button1.Location.X, Height);
                Height += 40;
            }
            else // if not a parent/admin, then hide admin button
			{
				button1.Enabled = false;
				button1.Visible = false;
			}
            this.ClientSize = new Size(this.ClientSize.Width, Height);
        }

		// Timer event to updated label text
		public void OnTimerTick(object sender, EventArgs eArgs)
		{
			if (sender == timer1)
			{
                UserInfo me = users.FindByName(Environment.UserName);
                Assert.Test(me != null);

                if (me.Active)
                {
                    me.AddSecond();

                    //
                    // Save to file every so often
                    //
                    if ((me.TotalTimeUsed % 30) == 0)
                    {
                        me.SaveToFile();
                    }
                }

                // show the time on the title bar
                this.Text = string.Format("pcNanny : {0} {1}", me.Name, me.ToString());

				//
				// Annoy user after we are over time limit by
				// bringing window to front every 3 minutes.
				//
				if (me.EnableNag &&
					me.IsOverTime() &&
					((me.TotalTimeUsed / 60) % 3) == 0 &&	// every 3 minutes
					(me.TotalTimeUsed % 60) < 5)	// annoy for 5 seconds
				{
					Rectangle bounds = Screen.PrimaryScreen.Bounds;
					this.Left = (bounds.Width - this.Width) / 2;
					this.Top = (bounds.Height - this.Height) / 2;
					this.WindowState = FormWindowState.Normal;
					this.TopMost = true;
					this.Show();
				}
				else if (this.TopMost)
				{
					// if not being annoying, allow other windows to come to the front
					this.TopMost = false;
				}

				GC.Collect();
				Invalidate();
			}
		}

		#region Drawing code
		private void DrawStringCentered(Graphics g, string str, ref float linePos)
		{
			Font font = new Font("Tahoma", 14); // Font("Lucida Sans Unicode", 14);
			float centerPos = ClientRectangle.Width / 2;
			SizeF size = g.MeasureString(str, font);
			g.DrawString(str, font, Brushes.Black, centerPos - (size.Width / 2), linePos);
			linePos += font.Height + 5;
		}

		private void DrawUser(Graphics g, UserInfo user, ref float linePos)
		{
			Font font = new Font("Tahoma", 10, FontStyle.Regular);

			float pct = (float)user.TotalTimeUsed / (user.GetTimeLimit()*60);
			Color color;
			if (pct < 0)
				pct = 0;
			if (pct > 1)
			{
				pct = 1;
				color = Color.Crimson;
			}
			else
			{
				color = Color.Green;
			}

			float centerPos = ClientRectangle.Width / 2;
			Rectangle r = new Rectangle((int)(centerPos - 25), (int)linePos, 50, 50);
            g.FillPie(new SolidBrush(Color.DarkGray), r, -90, 360);
            r.Offset(-2, -2); r.Inflate(-5, -5);
            g.FillPie(new SolidBrush(Color.LightGray), r, -90, 360);
            r.Inflate(-2, -2);
			g.FillPie(new SolidBrush(color), r, -90, 360 * pct);

			if (user.Name == Environment.UserName)
			{
				float angle = (float)(user.TotalTimeUsed%60) / 60;
				angle = angle * 360;

				SolidBrush brush = new SolidBrush(Color.FromArgb(128, 64, 128, 64));
                //HatchBrush brush = new HatchBrush(HatchStyle.Percent50,
                //    Color.White, Color.Transparent);
                g.FillPie(brush, r, angle - 90 + 45 - 10, 20);
				g.FillPie(brush, r, angle - 90 + 45 + 90 - 10, 20);
				g.FillPie(brush, r, angle - 90 + 45 + 180 - 10, 20);
				g.FillPie(brush, r, angle - 90 + 45 + 270 - 10, 20);
			}

			// draw user
			SizeF size = g.MeasureString(user.Name, font);
			g.DrawString(user.Name, font, Brushes.Black, centerPos - 30 - size.Width, linePos+(50-font.Height)/2);
			g.DrawString(user.ToString(), font, Brushes.Black, centerPos + 30, linePos + (50 - font.Height) / 2);
			linePos += 60;
		}

		protected override void OnPaint(PaintEventArgs e)
		{
			// It will be double buffered to stop flickering
			//Create Temporary Canvas
			System.Drawing.Bitmap timage = new Bitmap(this.Size.Width, this.Size.Height);

			//Create our Graphics Objects
			System.Drawing.Graphics g = Graphics.FromImage(timage);

			//Draw anything to the bitmap eg.
			g.FillRectangle(new SolidBrush(this.BackColor), 0, 0, this.Size.Width, this.Size.Height);

			float linePos = 10;
			//DrawStringCentered(e, "Friday, April 25, 2008", ref linePos);
			DrawStringCentered(g, DateTime.Today.ToString("dddd, MMMM d, yyyy"), ref linePos);
			linePos -= 10;
			DrawStringCentered(g, "~ Time used (limit) ~", ref linePos);
			linePos += 5;
            
            foreach (UserInfo user in users)
			{
				DrawUser(g, user, ref linePos);
			}

			e.Graphics.DrawImageUnscaled(timage, 0, 0, this.Size.Width, this.Size.Height);

			g.Dispose();

			GC.Collect();
		}
		#endregion

		#region Disable close window
		private static int WM_QUERYENDSESSION = 0x11;
		private static bool systemShutdown = false;

		protected override void WndProc(ref Message m)
		{
			if (m.Msg == WM_QUERYENDSESSION)
			{
				systemShutdown = true;
			}
			base.WndProc(ref m);
		}

		private void OnClosingEvent(object sender, System.ComponentModel.CancelEventArgs e)
		{
			if (!systemShutdown)
				e.Cancel = true;    // don't let the user close the window until logging off
		}
		#endregion

        #region SensLogon events
        private void SensLogon_DisplayLock(string userName)
        {
            Console.WriteLine("Screen Locked: " + userName);
            if (userName == Environment.UserName)
            {
                UserInfo user = users.FindByName(userName);
                user.ScreenLocked = true;
            }
        }
        private void SensLogon_DisplayUnlock(string userName)
        {
            Console.WriteLine("Screen Unlocked: " + userName);
            if (userName == Environment.UserName)
            {
                UserInfo user = users.FindByName(userName);
                user.ScreenLocked = false;
            }
        }
        private void SensLogon_StartScreenSaver(string userName)
        {
            Console.WriteLine("StartScreenSaver: " + userName);
            if (userName == Environment.UserName)
            {
                UserInfo user = users.FindByName(userName);
                user.ScreenSaved = true;
            }
        }
        private void SensLogon_StopScreenSaver(string userName)
        {
            Console.WriteLine("StopScreenSaver: " + userName);
            if (userName == Environment.UserName)
            {
                UserInfo user = users.FindByName(userName);
                user.ScreenSaved = false;
            }
        }
        #endregion

        /// <summary>
        /// Admin users
        /// </summary>
        private void button1_Click(object sender, EventArgs e)
		{
            UserInfo me = users.FindByName(Environment.UserName);
            Assert.Test(me != null);

            Form form = new PickUserForm(ref users);
            form.ShowDialog();
        }
	}
}