using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace pcNanny
{
    /// <summary>
    /// Simple form to edit the time limits and options for a User.
    /// </summary>
	public partial class Form2 : Form
	{
		UserInfo user = null;

        /// <summary>
        /// Form constructor copies the user info to the controls
        /// </summary>
        /// <param name="user">User to edit</param>
		public Form2(ref UserInfo user)
		{
			InitializeComponent();
            StartPosition = FormStartPosition.CenterParent;

			this.user = user;

            textBox8.Text = user.Name;

			textBox1.Text = user.GetTimeLimit(DayOfWeek.Monday).ToString();
			textBox2.Text = user.GetTimeLimit(DayOfWeek.Tuesday).ToString();
			textBox3.Text = user.GetTimeLimit(DayOfWeek.Wednesday).ToString();
			textBox4.Text = user.GetTimeLimit(DayOfWeek.Thursday).ToString();
			textBox5.Text = user.GetTimeLimit(DayOfWeek.Friday).ToString();
			textBox6.Text = user.GetTimeLimit(DayOfWeek.Saturday).ToString();
			textBox7.Text = user.GetTimeLimit(DayOfWeek.Sunday).ToString();

			checkBox1.Checked = user.EnableNag;
			checkBox2.Checked = user.EnableLog;
		}

        /// <summary>
        /// Sets all time limits to one hour.
        /// </summary>
		private void button4_Click(object sender, EventArgs e)
		{
			// all one hour
			textBox1.Text = "60";
			textBox2.Text = "60";
			textBox3.Text = "60";
			textBox4.Text = "60";
			textBox5.Text = "60";
			textBox6.Text = "60";
			textBox7.Text = "60";
		}

        /// <summary>
        /// View key log for user
        /// </summary>
		private void button3_Click(object sender, EventArgs e)
		{
			// view log
			user.OpenLog();
		}

        /// <summary>
        /// Cancel edit back to last save.
        /// </summary>
		private void button1_Click(object sender, EventArgs e)
		{
			// cancel
			Close();
		}

        /// <summary>
        /// Parse a time limit and catch errors.
        /// </summary>
		private int ParseTimeLimit(string s)
		{
			int result;
			try
			{
				result = int.Parse(s);
				result = MathHelper.Clamp(result, 0, 60 * 24);
			}
			catch (Exception)
			{
				result = 60;    // default is 60 minutes
			}
			return result;
		}

        /// <summary>
        /// Copy control info to user and save.
        /// </summary>
		private void button2_Click(object sender, EventArgs e)
		{
			// save
            user.SetTimeLimit(DayOfWeek.Monday, ParseTimeLimit(textBox1.Text));
            user.SetTimeLimit(DayOfWeek.Tuesday, ParseTimeLimit(textBox2.Text));
            user.SetTimeLimit(DayOfWeek.Wednesday, ParseTimeLimit(textBox3.Text));
            user.SetTimeLimit(DayOfWeek.Thursday, ParseTimeLimit(textBox4.Text));
            user.SetTimeLimit(DayOfWeek.Friday, ParseTimeLimit(textBox5.Text));
            user.SetTimeLimit(DayOfWeek.Saturday, ParseTimeLimit(textBox6.Text));
            user.SetTimeLimit(DayOfWeek.Sunday, ParseTimeLimit(textBox7.Text));

			user.EnableNag = checkBox1.Checked;
			user.EnableLog = checkBox2.Checked;

            user.SaveToFile();
        }
	}
}